import json
import os
import stat
import tempfile
from contextlib import contextmanager

import yaml


def read_json(filename):
    """
    Read json from file
    """
    return json.loads(read_file(filename))


def read_file(filename):
    """
    Read in a file content
    """
    with open(filename, "r") as fd:
        content = fd.read()
    return content


def write_file(content, filename, executable=False):
    with open(filename, "w") as fd:
        fd.write(content)

    # Make the file executable
    if executable:
        st = os.stat(filename)
        os.chmod(filename, st.st_mode | stat.S_IEXEC | stat.S_IXGRP | stat.S_IXOTH)


def read_yaml(filename):
    """
    Read yaml from file
    """
    with open(filename, "r") as fd:
        content = yaml.safe_load(fd)
    return content


def write_yaml(obj, filename):
    """
    Read yaml to file
    """
    with open(filename, "w") as fd:
        yaml.dump(obj, fd)


@contextmanager
def workdir(dirname):
    """
    Provide context for a working directory, e.g.,

    with workdir(name):
       # do stuff
    """
    here = os.getcwd()
    os.chdir(dirname)
    try:
        yield
    finally:
        os.chdir(here)


def get_tmpdir(tmpdir=None, prefix="", create=True):
    """
    Get a temporary directory for an operation.
    """
    tmpdir = tmpdir or tempfile.gettempdir()
    prefix = prefix or "flux-metrics-api-tmp"
    prefix = "%s.%s" % (prefix, next(tempfile._get_candidate_names()))
    tmpdir = os.path.join(tmpdir, prefix)

    if not os.path.exists(tmpdir) and create is True:
        os.mkdir(tmpdir)

    return tmpdir


def pretty_print_list(listing):
    """
    Pretty print a list of json (python dictionaries)
    """
    # Prepare common format
    if not isinstance(listing, list):
        listing = [listing]
    result = ""
    for i, item in enumerate(listing):
        result += "".join(f"({k}:{v})," for k, v in item.items())
        if len(listing) > 1 and i != len(listing) - 1:
            result += "\n"
    return result.strip(",")



from fluxgen.template import Template

# Optional arguments (with defaults)
# I don't see the need to expose these (will do when there is reason)
# - mamba_prefix
# - python_version
# - linkname (should be discovered if not provided)
# - command (should be added if being used)


class FluxInstallScript:
    # Required template variables
    # Note this isn't currently used
    required = ["lead-broker", "brokers"]

    # Defaults
    # Optional:
    # mamba_prefix

    def __init__(self):
        """
        Create a new installl script.

        The kwargs are expected to be variables for the template.
        """
        self.template = Template("install-flux.sh")

    def render(self, **vars):
        """
        Render the variables into the template
        """
        for varname in self.required:
            if varname not in vars or vars[varname] is None:
                raise ValueError(f"{varname} is required.")

            # Convert "-" to _ for jinja2
            if "-" in varname:
                updated = varname.replace("-", "_")
                vars[updated] = vars[varname]

        # Brokers needs to be list
        brokers = vars["brokers"]

        # Command should be a string
        command = vars.get("command")
        if command is not None and isinstance(command, list):
            vars["command"] = " ".join(command)

        if not isinstance(brokers, list):
            brokers = [x.strip() for x in brokers.split(",") if x.strip()]
            vars["brokers"] = brokers
        return self.template.render(**vars)

    @classmethod
    def get_required(cls):
        return cls.required



import os

from jinja2 import Environment, FileSystemLoader

here = os.path.dirname(os.path.abspath(__file__))

# Allow includes from this directory OR providing strings
template_dir = os.path.join(here, "templates")
env = Environment(loader=FileSystemLoader(template_dir))


class Template:
    """
    Supporting functions for loading a script template.
    """

    def __init__(self, template_file):
        self.get(template_file)

    def get(self, template_name):
        """
        Get a template from templates
        """
        template_file = os.path.join(here, "templates", template_name)
        if not os.path.exists(template_file):
            template_file = os.path.abspath(template_name)
        if not os.path.exists(template_file):
            raise ValueError(f"template file {template_file} does not exist.")
        self.template_file = template_file

    def render(self, **vars):
        """
        Load varibles into the template
        """
        with open(self.template_file, "r") as temp:
            template = env.from_string(temp.read())
        return template.render(vars)



#!/usr/bin/env python

import argparse
import sys

import fluxgen


def get_parser():
    parser = argparse.ArgumentParser(
        description="Fluxgen Python",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--version",
        dest="version",
        help="show software version.",
        default=False,
        action="store_true",
    )

    description = "actions for Fluxgen"
    subparsers = parser.add_subparsers(
        help="actions",
        title="actions",
        description=description,
        dest="command",
    )

    # print version and exit
    subparsers.add_parser("version", description="show software version")

    # Write the install and configuration script
    create = subparsers.add_parser(
        "create",
        description="create an install and configuration script for flux",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    create.add_argument(
        "--brokers",
        help="Brokers, either short hand or comma separated",
    )
    create.add_argument(
        "--linkname",
        help="Device name (e.g., eth0)",
        default="eth0",
    )
    create.add_argument(
        "--subdomain",
        help="Subdomain for the workers (e.g., m.default.svc.cluster.local)",
    )
    create.add_argument(
        "--outfile",
        help="Name for install file (defaults to flux-install.sh)",
        default="flux-install.sh",
    )
    create.add_argument(
        "--dry-run",
        help="Preview the install script (don't write to file)",
        default=False,
        action="store_true",
    )
    create.add_argument(
        "--lead-broker",
        help="Is this the lead broker?",
        default=False,
        action="store_true",
    )
    return parser


def run_fluxgen():
    parser = get_parser()

    def help(return_code=0):
        version = fluxgen.__version__

        print("\nFluxgen Python v%s" % version)
        parser.print_help()
        sys.exit(return_code)

    # If the user didn't provide any arguments, show the full help
    if len(sys.argv) == 1:
        help()

    # If an error occurs while parsing the arguments, the interpreter will exit with value 2
    args, command = parser.parse_known_args()

    # Show the version and exit
    if args.command == "version" or args.version:
        print(fluxgen.__version__)
        sys.exit(0)

    # retrieve subparser (with help) from parser
    helper = None
    subparsers_actions = [
        action for action in parser._actions if isinstance(action, argparse._SubParsersAction)
    ]
    for subparsers_action in subparsers_actions:
        for choice, subparser in subparsers_action.choices.items():
            if choice == args.command:
                helper = subparser
                break

    # Does the user want a shell?
    if args.command == "create":
        from .create import main

    # Pass on to the correct parser
    return_code = 0
    try:
        main(args=args, parser=parser, command=command, subparser=helper)
        sys.exit(return_code)
    except UnboundLocalError:
        return_code = 1
    help(return_code)


if __name__ == "__main__":
    run_fluxgen()
